local Settings = LibStub:NewLibrary("SomeCooldowns-Settings", 1)
if not Settings then return end

local D = LibStub:GetLibrary("DoomCore-1.3")

local SharedMedia = LibStub( "LibSharedMedia-3.0" )

local defaults = {}
defaults.global = {
  background = "None",
  border = "None",
  lock = true,
  displayItems = true,
  displaySpells = false,
  displayToys = true,
  reverse = true,
  anchor = "bottom",
  sort = "long",
  offsetX = 485,
  offsetY = 120,
  grow = "left",
  padding = 0,
  edge = 4,
  text = false,
  limit = 24,
  max = 0,
  color = { r=0, g=0, b=0, a=1 },
  recharge = true,
  rowGrowth = "top",
  columnGrowth = "right",
  iconSize = 38,
  spacing = 4,
  tooltip = true,
  tooltipOverride = false,
  tooltipAnchor = "topright"
}
defaults.db = {
  profile = {
  }
}
defaults.options = {
  type = "group",
  get = "ConfGet",
  set = "Rebuild",
  args = {
    display = {
      type = "description",
      name = "Display:",
      fontSize = "medium",
      width = "half",
      order = 0
    },
    displayItems = {
      type = "toggle",
      name = "Items",
      width = "half",
      set = "SetFilter",
      order = 1,
    },
    displaySpells = {
      type = "toggle",
      name = "Spells",
      width = "half",
      set = "SetFilter",
      order = 2
    },
    displayToys = {
      type = "toggle",
      name = "Toys",
      set = "SetFilter",
      order = 3
    },
    sort = {
      name = "Start with",
      type = "select",
      values = {
        short = "Shortest",
        long = "Longest"
      },
      width = "half",
      order = 5
    },
    space1 = {
      type = "description",
      name = "",
      width = "full",
      order = 6
    },
    max = {
      type = "range",
      name = "Max buttons",
      min = 0,
      softMax = 48,
      step = 1,
      desc = "0 = Unlimited",
      order = 9
    },
    grow = {
      type = "select",
      name = "Grow",
      values = D.grow,
      width = "half",
      order = 10
    },
    limit = {
      type = "range",
      name = function()
        if D.direction[Settings.handler:ConfGet{ "grow" }] == D.HORIZONTAL then return "Max per row"
        else return "Max per column" end
      end,
      min = 0,
      softMax = 48,
      step = 1,
      desc = "0 = Unlimited",
      order = 11
    },
    rowGrowth = {
      type = "select",
      name = "Grow rows",
      values = {
        top = "Up",
        bottom = "Down"
      },
      hidden = function()
        local limit = tonumber(Settings.handler:ConfGet{ "limit" })
        if not limit or limit <= 0 then return true end
        return D.direction[Settings.handler:ConfGet{ "grow" }] ~= D.HORIZONTAL
      end,
      width = "half",
      order = 20
    },
    columnGrowth = {
      type = "select",
      name = "Grow columns",
      values = {
        right = "Right",
        left = "Left"
      },
      hidden = function()
        local limit = tonumber(Settings.handler:ConfGet{ "limit" })
        if not limit or limit <= 0 then return true end
        return D.direction[Settings.handler:ConfGet{ "grow" }] ~= D.VERTICAL
      end,
      width = "half",
      order = 21
    },
    space2 = {
      type = "description",
      name = "",
      width = "full",
      order = 22
    },
    iconSize = {
      type = "range",
      name = "Icon size",
      min = 1,
      max = 500,
      softMin = 10,
      softMax = 50,
      step = 0.5,
      order = 29
    },
    spacing = {
      type = "range",
      name = "Spacing",
      min = 1,
      max = 500,
      softMin = 10,
      softMax = 50,
      step = 0.5,
      desc = "Spacing between icons",
      order = 30
    },
    space3 = {
      type = "description",
      name = "",
      width = "full",
      order = 31
    },
    recharge = {
      type = "toggle",
      name = "Show recharges",
      desc = "Show spells with charges remaining",
      set = "SetText",
      order = 32
    },
    text = {
      type = "toggle",
      name = "Show numbers",
      desc = "Show time remaining on cooldowns",
      set = "SetText",
      order = 33
    },
    color = {
      type = "color",
      name = "Shading",
      set = "SetSwipeColor",
      hasAlpha = true,
      desc = "Color of the shaded portion of cooldowns.",
      width = "half",
      order = 34
    },
    reverse = {
      type = "toggle",
      name = "Reverse shading",
      desc = "Reverse the shaded portion of cooldowns",
      set = "SetReverse",
      order = 35
    },
    space4 = {
      type = "description",
      name = "",
      width = "full",
      order = 39
    },
    background = {
      name = "Background",
      type = "select",
      dialogControl = "LSM30_Background",
      values = SharedMedia:HashTable( SharedMedia.MediaType.BACKGROUND ),
      order = 40
    },
    padding = {
      type = "range",
      name = "Padding",
      desc = "Padding around edge of window",
      min = 0,
      max = 100,
      softMax = 24,
      step = 0.5,
      order = 41
    },
    space10 = {
      type = "description",
      name = "",
      width = "full",
      order = 42
    },
    border = {
      name = "Border",
      type = "select",
      dialogControl = "LSM30_Border",
      values = SharedMedia:HashTable( SharedMedia.MediaType.BORDER ),
      order = 43
    },
    edge = {
      type = "range",
      min = 1,
      max = 50,
      softMin = 1,
      softMax = 24,
      name = "Border size",
      order = 44
    },
    space5 = {
      type = "description",
      name = "",
      width = "full",
      order = 50
    },
    anchor = {
      type = "select",
      name = "Anchor",
      values = D.anchors,
      order = 60
    },
    lock = {
      type = "toggle",
      name = "Lock",
      order = 61
    },
    space11 = {
      type = "description",
      name = "",
      width = "full",
      order = 62
    },
    offsetX = {
      type = "range",
      name = "X",
      min = -UIParent:GetWidth(),
      max = UIParent:GetWidth(),
      step = 0.5,
      disabled = "Locked",
      order = 63
    },
    offsetY = {
      type = "range",
      name = "Y",
      min = -UIParent:GetHeight(),
      max = UIParent:GetHeight(),
      step = 0.5,
      disabled = "Locked",
      order = 64
    },
    space6 = {
      type = "description",
      name = "",
      width = "full",
      order = 70
    },
    tooltip = {
      type = "toggle",
      name = "Show Tooltips",
      set = "UpdateTooltips",
      order = 71
    },
    tooltipOverride = {
      type = "toggle",
      name = "Override Anchor",
      set = "UpdateTooltips",
      disabled = function() return not Settings.handler:ConfGet{ "tooltip" } end,
      order = 72
    },
    tooltipAnchor = {
      type = "select",
      name = "Anchor",
      set = "UpdateTooltips",
      values = D.anchors,
      disabled = function()
        return not Settings.handler:ConfGet{ "tooltip" } or not Settings.handler:ConfGet{ "tooltipOverride" }
      end,
      order = 73
    },
    whitelist = {
      type = "group",
      inline = true,
      name = "Whitelist",
      desc = "Always track",
      args = { },
      order = 101
    },
    addToWhitelist = {
      type = "input",
      name = "Add To Whitelist",
      set = "Add",
      width = "full",
      order = 102
    },
    blacklist = {
      type = "group",
      inline = true,
      name = "Blacklist",
      desc = "Always ignore",
      args = { },
      order = 300
    },
    addToBlacklist = {
      type = "input",
      name = "Add To Blacklist",
      width = "full",
      set = "Add",
      order = 302
    }
  }
}
Settings.defaults = defaults

function Settings:Add(group, val)
  local entry = {
    type = "execute",
    func = function()
      self.handler:ConfDelete{"group", val }
      if type(val) == "number" then
        self.handler.cooldowns.item[val] = nil
      else
        self.handler.cooldowns.spell[val] = nil
      end
      self.crawler:Set({ group, "args", tostring(val) }, nil)

      self.handler:Update(group == "whitelist")
    end
  }
  if type(val) == "number" then
    entry.name = GetItemInfo(val)
    entry.image = GetItemIcon(val)
  else
    entry.name = val
    entry.image = select(3, GetSpellInfo(val))
  end
  if not entry.name then return end
  entry.order = D.orderNum(entry.name)

  self.crawler:Set({ group, "args", tostring(val) }, entry)
end

Settings.debug = {
  somebars = {
    type = "toggle",
    name = "Blacklist all from Some Bars",
    hidden = function() return not Settings.handler.lib.somebars end,
    width = "full",
    order = 10
  },
}